#!/usr/bin/env Rscript

library(R0)
library(readr)
library(RCurl)
library(tidyverse)
library(lubridate)

# date formats
f1<-"%Y-%m-%d"
f2<-"%m/%d/%Y"

# Make sure you're in the repository containing all the data files

getDataByCountry<-function(data,country){
	indx<-which(data$'Country/Region' == country)
	indx
}

getSI<-function(classification){
	# classification = {"Probable","Certain","all"}
	serial.interval.file<-"serialIntervalData.csv"
	serial.interval<-as.data.frame(read_csv(serial.interval.file))
	serial.interval$ER<-as.integer(serial.interval$ER)
	serial.interval$EL<-as.integer(serial.interval$EL)
	serial.interval$SL<-as.integer(serial.interval$SL)
	serial.interval$SR<-as.integer(serial.interval$SR)
	serial.interval$InfecteeOnset<-as.Date(serial.interval$InfecteeOnset,f2)
	serial.interval$InfectorOnset<-as.Date(serial.interval$InfectorOnset,f2)

	if(classification != "all"){
		indx<-which(serial.interval$SIClassification == classification)
		serial.interval<-serial.interval[indx,]
	}

	serial.interval.tmp<-data.frame(InfectorOnset=serial.interval$InfectorOnset,
									InfecteeOnset=serial.interval$InfecteeOnset)
	SI<-as.vector(serial.interval.tmp$InfecteeOnset - serial.interval.tmp$InfectorOnset)

	SI
}


createTimeSeriesDataCountry<-function(country){
  confirmed.file<-"time_series_19-covid-Confirmed.csv"
  deaths.file<-"time_series_19-covid-Deaths.csv"
  recovered.file<-"time_series_19-covid-Recovered.csv"
  
  confirmed<-read_csv(confirmed.file)
  deaths<-read_csv(deaths.file)
  recovered<-read_csv(recovered.file)
  
  indx.conf<-getDataByCountry(confirmed,country)
  indx.death<-getDataByCountry(deaths, country)
  indx.recov<-getDataByCountry(recovered,country)
  
  len<-length(confirmed)
  data<-data.frame(date=as.Date(names(confirmed[5:length(confirmed)]),"%m/%d/%y"),
                   confirmed=unlist(lapply(seq(5,len,1), function(x) sum(confirmed[indx.conf,x]))),
                   recovered=unlist(lapply(seq(5,len,1), function(x) sum(recovered[indx.recov,x]))),
                   deaths=unlist(lapply(seq(5,len,1), function(x) sum(deaths[indx.death,x])))
  )
  tmp<-length(data$confirmed)
  TS<-data.frame(date=data$date,
                 incidence=c(0,rev(unlist(lapply(seq(tmp,2,-1) , function(x) data$confirmed[x]-data$confirmed[x-1])))))
  
  TS
}

getGenerationTime<-function(classification){
	# classification = {"Probable","Certain","all"}
	# For the COVID-19 data, the best fitting GT distribution is always a lognormal distribution
	# Mean and sd vary a little given the type of classification
	SI<-getSI(classification)
	gt<-est.GT(serial.interval=SI)

	# Create a GT class object for the R0 estimation 
	gt.c<-generation.time("lognormal",c(gt$mean,gt$sd))
	gt.c
}

createDataForR0est<-function(country,source,type_of_data){
  # source of data = {"WHO", "CSSE"}
  if(source=="SPF"){
    TS<-getSPFdata(country,type_of_data)
  }

	tmp<-c(as.numeric(as.character(TS$incidence)))
	names(tmp)<-c(format(as.Date(as.character(TS$date)),f1))
	len<-length(tmp)
	tot<-length(which(tmp !=0))

	# Check that first incidence data is not null, otherwise you'll get an error
	# Get the first index of non-nul incidence data
	# And remove the last incidence data if null 
	first<-(which(tmp != 0)[[1]])
	end<-(which(tmp !=0)[[tot]])
	tmp<-tmp[first:end]

	tmp
}


estimateReTimeDependant<-function(country,classification,source,type_of_data){
	# Estimation of the R(t)
	data<-createDataForR0est(country,source,type_of_data)
	gt<-getGenerationTime(classification)
	start<-as.Date(names(data[1]),f1)
	end<-as.Date(names(data[length(data)]),f1)

	R0.TD<-est.R0.TD(epid=data,
			 	 GT=gt,
		   		 begin=start,
		   		 end=end,
		   		 nsim=5000)

	R0.TD
}



getSPFdata<-function(country,type_of_data){
  
  
  # Data --------------------------------------------------------------------
  
  
  dataSPF <- read_csv("https://github.com/opencovid19-fr/data/raw/master/dist/chiffres-cles.csv")
  
  date_epidemie <- dataSPF %>% pull(date) %>% unique()
  ## À partir 2020-03-18, on ne prend plus que "Santé publique Fracne Data" comme source.
  data_av <- dataSPF %>% filter(date <= "2020-03-17", source_type == "sante-publique-france")
  data_ap <- dataSPF %>% filter(date >= "2020-03-18", source_type == "sante-publique-france-data",
                             maille_nom != "France")
  data_ap_fr <- dataSPF %>% filter(date >= "2020-03-18",
                                source_type == "ministere-sante",
                                maille_nom == "France")
  
  data2 <- bind_rows(data_av, data_ap, data_ap_fr)
  

  # Datasets trajectoires granularié / dates --------------------------------
  
  hospitalisation <- data2 %>%
    select(date, maille_nom, hospitalises) %>%
    distinct() %>%
    pivot_wider(names_from = maille_nom,
                values_from = hospitalises,
                values_fill = list(NA))
  
  deces <- data2 %>%
    select(date, maille_nom, deces) %>%
    distinct() %>%
    pivot_wider(names_from = maille_nom,
                values_from = deces,
                values_fill = list(NA))
  
  infectes <- data2 %>%
    select(date, maille_nom, cas_confirmes) %>%
    distinct() %>%
    pivot_wider(names_from = maille_nom,
                values_from = cas_confirmes,
                values_fill = list(NA))
  
  recovered <- data2 %>%
    select(date, maille_nom, gueris) %>%
    distinct() %>%
    pivot_wider(names_from = maille_nom,
                values_from = gueris,
                values_fill = list(NA))
  
  
  # Département numéro ------------------------------------------------------
  
  dpts_names <- read.csv(file="departements-francais.csv", sep = "\t")[, 1:2]
  N0_dpt <- c(656955,            526050,            331315,            165197,
              141756,            1079396,           326875,            265531,
              152398,            309907,            372705,            278360,
              2034469,           691453,            142811,            348180,
              647080,            296404,            240336,            162421,
              182258,            532886,            596186,            116270,
              408393,            539449,            520560,            600687,
              429425,            906554,            748468,            1400935,
              190040,            1633440,           1176145,           1082073,
              217139,            605380,            1264979,           257849,
              411979,            327835,            764737,            226901,
              1437137,           682890,            173166,            330336,
              76286,             815881,            490669,            563823,
              169250,            305365,            730398,            181641,
              755566,            1035866,           199596,            2588988,
              825077,            276903,            1452778,           660240,
              683169,            226839,            479000,            1132607,
              763204,            1876051,           233194,            547824,
              560227,            432548,            828405,            2148271,
              1243788,           1423607,           1448625,           372627,
              569769,            387898,            262618,            1073836,
              560997,            683187,            437398,            370774,
              359520,            332096,            140145,            1319401,
              1613762,           1670149,           1406041,           1248354,
              376879,            358749,            290691,            859959,
              279471)
  dpts_names$N0 <- N0_dpt
  dpts_names$NUMeRO <- as.character(dpts_names$NUMÉRO)
  
  
  list_dpts <- c(0, dpts_names$NUMeRO)
  names(list_dpts) <- c("France", paste(dpts_names$NUMeRO, "-", dpts_names$NOM))
  
  
  # Incidence hosto ---------------------------------------------------------
  

  
  ## avant le 2020-03-18
  data_recovered_av <- dataSPF %>% filter(date < "2020-03-18",
                                       maille_nom == "France",
                                       source_nom == "Ministère des Solidarités et de la Santé")
  
  ## à partir du 2020-03-18
  data_recovered_ap <- dataSPF %>% filter(date >= "2020-03-18",
                                       maille_nom == "France",
                                       source_nom == "OpenCOVID19-fr")
  
  data_recovered <- bind_rows(data_recovered_av, data_recovered_ap)
  
  ### Pas prendre les gars en réa mais les gars hospitalisés.
  
  temp <- tibble(date = date_epidemie)
  temp$hosp <- NA
  temp$hosp[date_epidemie %in% hospitalisation$date] <- hospitalisation$France
  
  temp$deces <- NA
  temp$deces[date_epidemie %in% deces$date] <- deces$France
  
  temp$reco <- NA
  temp$reco[date_epidemie %in% data_recovered$date] <- data_recovered$gueris
  
  ## new H = (Ht - Ht-1) - (Dt-1 - Dt-2) - (Rt-1 - Rt-2)
  
  incidence_hosp <- tibble(date = date_epidemie,
                           incidence = NA)
  incidence_hosp$incidence[-1] <- diff(temp$hosp) +
    diff(temp$deces) +
    diff(temp$reco)
  
  incidence_hosp$incidence[is.na(incidence_hosp$incidence)]<-0
  incidence_hosp$incidence[incidence_hosp$incidence<2]<-0
 # incidence_hosp<-tail(incidence_hosp,-(max(which(incidence_hosp$incidence==0)-1)))
  final_data_hospitalise<-cbind(format(as.Date(incidence_hosp$date),f1),incidence_hosp$incidence)
  
  ### Getting number for the dead
  
  deces <- data2 %>%
    select(date, maille_nom, deces) %>%
    distinct() %>%
    pivot_wider(names_from = maille_nom,
                values_from = deces,
                values_fill = list(NA))
  
  
  deces_France <- deces[,c(1,3)]
  names(deces_France)<-c("date","incidence")
  deces_France$incidence[is.na(deces_France$incidence)]<-0
  deces_France$incidence<-deces_France$incidence-c(0,head(deces_France$incidence,-1))
#  deces_France$incidence[deces_France$incidence<2]<-0
#  deces_France<-tail(deces_France,-(max(which(deces_France$incidence==0)-1)))
  final_data_deces<-cbind(format(as.Date(deces_France$date),f1),deces_France$incidence)
  
  
  final_data<-c()

  i.type_of_data<-type_of_data
  
  if(as.character(i.type_of_data)=="mortalite")
  {
    final_data<-final_data_deces
  }
  else
  {
    if(as.character(i.type_of_data)=="hospitalisation")
    {
      final_data<-final_data_hospitalise
    }
    else
    {
      print("soucis avec l'origine des données")
    }
  }
  
  
  data<-as.data.frame(final_data)
  names(data)<-c("date","incidence")
  
  
  data$date<-as.Date(as.character(data$date))
  data$incidence<-as.numeric(as.character(data$incidence))  
  
  data
}


estimateR0ML<-function(country,classification,start,end,source){
  # Here you estimate the R0 using ML method
  # EX: start = "2020-01-24"
  # EX: end = "2020-03-10"
  data<-createDataForR0est(country,source,type_of_data)
  gt<-getGenerationTime(classification)
  R0.ML<-est.R0.ML(epid=data,
                   GT=gt,
                   begin=as.Date(start,f1),
                   end=as.Date(end,f1),
                   nsim=5000)
  
}


getWHOIncidenceFromCountry<-function(country){
  # Data from WHO taken from : https://covid.ourworldindata.org/data/new_cases.csv
  myfile<-getURL("https://covid.ourworldindata.org/data/ecdc/new_deaths.csv",ssl.verifyhost=FALSE,ssl.verifypeer=FALSE)
  new.cases<-read.csv(file = textConnection(myfile),header = T)
  #new.cases<-read.csv(file = "covid_new_cases.csv")
  new.cases.country<-new.cases[country]
  new.cases.country[is.na(new.cases.country)]<-0
  names(new.cases.country)<-"incidence"
  dates<-as.vector(new.cases$date)
  len<-length(new.cases.country)
  data<-data.frame(date=as.Date(dates,f1),
                   incidence=new.cases.country)
  data
}



getScale <- function(scale)#Internal scaling function to display proper X-Axis labels
  ###Internal scaling function to display proper X-Axis labels
{
  ##keyword<< internal
  #Scale parameters are used to adjust dates on X-Axis labels
  if (scale == "d") {
    div = 1
  } else if (scale == "w") {
    div = 7
  } else if (scale == "f") {
    div = 15
  } else if (scale == "m") {
    div = 30
  } else {
    stop("Invalid scale parameter.")
  }
  return(div)
}

plotR0TD<-function(x,xscale){
  R = x$R
  epid = x$epid
  epid$t = epid$t[x$begin.nb:(x$end.nb-1)]
  polygon.x = c(epid$t, rev(epid$t))
  polygon.y = c(x$conf.int[1:length(epid$t),1], rev(x$conf.int[1:length(epid$t),2]))
  
  div = get.scale(xscale)
  atLab = pretty(epid$t, n=length(epid$t)/div)
  lab = format(pretty(epid$t, n=length(epid$t)/div),"%d %b")
  
  plot(epid$t, R[1:length(epid$t)], ylim=c(0, max(polygon.y)), xlab="Temps", ylab="R(t)", xaxt="n", pch=NA_integer_, lty="blank", main="Nombre de reproduction efficace")
  polygon(polygon.x, polygon.y, col="gray", border=NA)
  lines(epid$t, R[1:length(epid$t)])
  abline(h=1, lty="dashed", col="gray40")
  
  axis(1, at=atLab, labels=lab)
  
}

